@@ -4,23 +4,23 @@ use pyo3::types::PyType;
 use pyo3::{prelude::*, PyMappingProtocol, PyObjectProtocol};
 
 use crate::monomorphs::RaggedBufferI64;
-use crate::ragged_buffer::RaggedBuffer;
+use crate::ragged_buffer_view::RaggedBufferView;
 
-use super::{IndicesOrInt, PadpackResult};
+use super::{Index, MultiIndex, PadpackResult};
 
 #[pyclass]
 #[derive(Clone)]
-pub struct RaggedBufferF32(pub RaggedBuffer<f32>);
+pub struct RaggedBufferF32(pub RaggedBufferView<f32>);
 
 #[pymethods]
 impl RaggedBufferF32 {
     #[new]
     pub fn new(features: usize) -> Self {
-        RaggedBufferF32(RaggedBuffer::new(features))
+        RaggedBufferF32(RaggedBufferView::new(features))
     }
     #[classmethod]
     fn from_array(_cls: &PyType, array: PyReadonlyArray3<f32>) -> Self {
-        RaggedBufferF32(RaggedBuffer::from_array(array))
+        RaggedBufferF32(RaggedBufferView::from_array(array))
     }
     #[classmethod]
     fn from_flattened(
@@ -28,14 +28,13 @@ impl RaggedBufferF32 {
         flattened: PyReadonlyArray2<f32>,
         lengths: PyReadonlyArray1<i64>,
     ) -> PyResult<Self> {
-        Ok(RaggedBufferF32(RaggedBuffer::from_flattened(
+        Ok(RaggedBufferF32(RaggedBufferView::from_flattened(
             flattened, lengths,
         )?))
     }
     fn push(&mut self, items: PyReadonlyArrayDyn<f32>) -> PyResult<()> {
         if items.ndim() == 1 && items.len() == 0 {
-            self.0.push_empty();
-            Ok(())
+            self.0.push_empty()
         } else if items.ndim() == 2 {
             self.0.push(
                 &items
@@ -48,12 +47,12 @@ impl RaggedBufferF32 {
             ))
         }
     }
-    fn push_empty(&mut self) {
-        self.0.push_empty();
+    fn push_empty(&mut self) -> PyResult<()> {
+        self.0.push_empty()
     }
 
-    fn clear(&mut self) {
-        self.0.clear();
+    fn clear(&mut self) -> PyResult<()> {
+        self.0.clear()
     }
 
     fn as_array<'a>(
@@ -72,7 +71,7 @@ impl RaggedBufferF32 {
     fn size1(&self, py: Python, i: Option<usize>) -> PyResult<PyObject> {
         match i {
             Some(i) => self.0.size1(i).map(|s| s.into_py(py)),
-            None => Ok(self.0.lengths(py).into_py(py)),
+            None => self.0.lengths(py).map(|ok| ok.into_py(py)),
         }
     }
     fn size2(&self) -> usize {
@@ -86,22 +85,30 @@ impl RaggedBufferF32 {
     }
     #[classmethod]
     fn cat(_cls: &PyType, buffers: Vec<PyRef<RaggedBufferF32>>, dim: usize) -> PyResult<Self> {
-        Ok(RaggedBufferF32(RaggedBuffer::cat(
+        Ok(RaggedBufferF32(RaggedBufferView::cat(
             &buffers.iter().map(|b| &b.0).collect::<Vec<_>>(),
             dim,
         )?))
     }
     #[allow(clippy::type_complexity)]
     fn padpack<'a>(&self, py: Python<'a>) -> PadpackResult<'a> {
-        match self.0.padpack() {
+        match self.0.padpack()? {
             Some((padbpack_index, padpack_batch, padpack_inverse_index, dims)) => Ok(Some((
                 padbpack_index.to_pyarray(py).reshape(dims)?,
                 padpack_batch.to_pyarray(py).reshape(dims)?,
-                padpack_inverse_index.to_pyarray(py).reshape(self.0.len())?,
+                padpack_inverse_index
+                    .to_pyarray(py)
+                    .reshape(self.0.len()?)?,
             ))),
             _ => Ok(None),
         }
     }
+    fn items(&self) -> PyResult<usize> {
+        self.0.items()
+    }
+    fn clone(&self) -> Self {
+        RaggedBufferF32(self.0.clone())
+    }
 }
 
 #[pyproto]
@@ -131,6 +138,7 @@ pub enum RaggedBufferF32OrF32 {
     Scalar(f32),
 }
 
+// TODO: pass by reference?
 #[cfg(all())]
 #[pyproto]
 impl pyo3::PyNumberProtocol for RaggedBufferF32 {
@@ -140,7 +148,7 @@ impl pyo3::PyNumberProtocol for RaggedBufferF32 {
                 lhs.0.binop::<crate::ragged_buffer::BinOpAdd>(&rhs.0)?,
             )),
             RaggedBufferF32OrF32::Scalar(rhs) => Ok(RaggedBufferF32(
-                lhs.0.op_scalar::<crate::ragged_buffer::BinOpAdd>(rhs),
+                lhs.0.op_scalar::<crate::ragged_buffer::BinOpAdd>(rhs)?,
             )),
         }
     }
@@ -150,18 +158,33 @@ impl pyo3::PyNumberProtocol for RaggedBufferF32 {
                 lhs.0.binop::<crate::ragged_buffer::BinOpMul>(&rhs.0)?,
             )),
             RaggedBufferF32OrF32::Scalar(rhs) => Ok(RaggedBufferF32(
-                lhs.0.op_scalar::<crate::ragged_buffer::BinOpMul>(rhs),
+                lhs.0.op_scalar::<crate::ragged_buffer::BinOpMul>(rhs)?,
             )),
         }
     }
+
+    fn __isub__(&mut self, rhs: RaggedBufferF32) -> PyResult<()> {
+        self.0.binop_mut::<crate::ragged_buffer::BinOpSub>(&rhs.0)
+    }
 }
 
 #[pyproto]
 impl<'p> PyMappingProtocol for RaggedBufferF32 {
-    fn __getitem__(&self, index: IndicesOrInt<'p>) -> PyResult<RaggedBufferF32> {
+    fn __getitem__(&self, index: MultiIndex<'p>) -> PyResult<RaggedBufferF32> {
         match index {
-            IndicesOrInt::Indices(indices) => Ok(RaggedBufferF32(self.0.swizzle(indices)?)),
-            IndicesOrInt::Int(i) => Ok(RaggedBufferF32(self.0.get(i))),
+            MultiIndex::Index1(index) => match index {
+                Index::PermutationNP(indices) => Ok(RaggedBufferF32(self.0.swizzle(indices)?)),
+                Index::Permutation(_indices) => panic!("oh no"), //Ok(RaggedBufferF32(self.0.swizzle(indices)?)),
+                Index::Int(i) => Ok(RaggedBufferF32(self.0.get_sequence(i)?)),
+                Index::Slice(slice) => panic!("{:?}", slice),
+            },
+            MultiIndex::Index3((i0, i1, i2)) => Ok(RaggedBufferF32(Python::with_gil(|py| {
+                self.0.get_slice(py, i0, i1, i2)
+            })?)),
+            x => panic!("{:?}", x),
         }
     }
+    fn __len__(&self) -> PyResult<usize> {
+        self.0.len()
+    }
 }
