@@ -3,23 +3,23 @@ use pyo3::basic::CompareOp;
 use pyo3::types::PyType;
 use pyo3::{prelude::*, PyMappingProtocol, PyObjectProtocol};
 
-use crate::ragged_buffer::RaggedBuffer;
+use crate::ragged_buffer_view::RaggedBufferView;
 
-use super::{IndicesOrInt, PadpackResult};
+use super::{Index, MultiIndex, PadpackResult};
 
 #[pyclass]
 #[derive(Clone)]
-pub struct RaggedBufferI64(pub RaggedBuffer<i64>);
+pub struct RaggedBufferI64(pub RaggedBufferView<i64>);
 
 #[pymethods]
 impl RaggedBufferI64 {
     #[new]
     pub fn new(features: usize) -> Self {
-        RaggedBufferI64(RaggedBuffer::new(features))
+        RaggedBufferI64(RaggedBufferView::new(features))
     }
     #[classmethod]
     fn from_array(_cls: &PyType, array: PyReadonlyArray3<i64>) -> Self {
-        RaggedBufferI64(RaggedBuffer::from_array(array))
+        RaggedBufferI64(RaggedBufferView::from_array(array))
     }
     #[classmethod]
     fn from_flattened(
@@ -27,14 +27,13 @@ impl RaggedBufferI64 {
         flattened: PyReadonlyArray2<i64>,
         lengths: PyReadonlyArray1<i64>,
     ) -> PyResult<Self> {
-        Ok(RaggedBufferI64(RaggedBuffer::from_flattened(
+        Ok(RaggedBufferI64(RaggedBufferView::from_flattened(
             flattened, lengths,
         )?))
     }
     fn push(&mut self, items: PyReadonlyArrayDyn<i64>) -> PyResult<()> {
         if items.ndim() == 1 && items.len() == 0 {
-            self.0.push_empty();
-            Ok(())
+            self.0.push_empty()
         } else if items.ndim() == 2 {
             self.0.push(
                 &items
@@ -47,12 +46,12 @@ impl RaggedBufferI64 {
             ))
         }
     }
-    fn push_empty(&mut self) {
-        self.0.push_empty();
+    fn push_empty(&mut self) -> PyResult<()> {
+        self.0.push_empty()
     }
 
-    fn clear(&mut self) {
-        self.0.clear();
+    fn clear(&mut self) -> PyResult<()> {
+        self.0.clear()
     }
 
     fn as_array<'a>(
@@ -71,7 +70,7 @@ impl RaggedBufferI64 {
     fn size1(&self, py: Python, i: Option<usize>) -> PyResult<PyObject> {
         match i {
             Some(i) => self.0.size1(i).map(|s| s.into_py(py)),
-            None => Ok(self.0.lengths(py).into_py(py)),
+            None => self.0.lengths(py).map(|ok| ok.into_py(py)),
         }
     }
     fn size2(&self) -> usize {
@@ -85,18 +84,20 @@ impl RaggedBufferI64 {
     }
     #[classmethod]
     fn cat(_cls: &PyType, buffers: Vec<PyRef<RaggedBufferI64>>, dim: usize) -> PyResult<Self> {
-        Ok(RaggedBufferI64(RaggedBuffer::cat(
+        Ok(RaggedBufferI64(RaggedBufferView::cat(
             &buffers.iter().map(|b| &b.0).collect::<Vec<_>>(),
             dim,
         )?))
     }
     #[allow(clippy::type_complexity)]
     fn padpack<'a>(&self, py: Python<'a>) -> PadpackResult<'a> {
-        match self.0.padpack() {
+        match self.0.padpack()? {
             Some((padbpack_index, padpack_batch, padpack_inverse_index, dims)) => Ok(Some((
                 padbpack_index.to_pyarray(py).reshape(dims)?,
                 padpack_batch.to_pyarray(py).reshape(dims)?,
-                padpack_inverse_index.to_pyarray(py).reshape(self.0.len())?,
+                padpack_inverse_index
+                    .to_pyarray(py)
+                    .reshape(self.0.len()?)?,
             ))),
             _ => Ok(None),
         }
@@ -130,6 +131,7 @@ pub enum RaggedBufferI64OrI64 {
     Scalar(i64),
 }
 
+// TODO: pass by reference?
 #[cfg(all())]
 #[pyproto]
 impl pyo3::PyNumberProtocol for RaggedBufferI64 {
@@ -139,7 +141,7 @@ impl pyo3::PyNumberProtocol for RaggedBufferI64 {
                 lhs.0.binop::<crate::ragged_buffer::BinOpAdd>(&rhs.0)?,
             )),
             RaggedBufferI64OrI64::Scalar(rhs) => Ok(RaggedBufferI64(
-                lhs.0.op_scalar::<crate::ragged_buffer::BinOpAdd>(rhs),
+                lhs.0.op_scalar::<crate::ragged_buffer::BinOpAdd>(rhs)?,
             )),
         }
     }
@@ -149,18 +151,30 @@ impl pyo3::PyNumberProtocol for RaggedBufferI64 {
                 lhs.0.binop::<crate::ragged_buffer::BinOpMul>(&rhs.0)?,
             )),
             RaggedBufferI64OrI64::Scalar(rhs) => Ok(RaggedBufferI64(
-                lhs.0.op_scalar::<crate::ragged_buffer::BinOpMul>(rhs),
+                lhs.0.op_scalar::<crate::ragged_buffer::BinOpMul>(rhs)?,
             )),
         }
     }
+
+    fn __isub__(&mut self, rhs: RaggedBufferI64) -> PyResult<()> {
+        self.0.binop_mut::<crate::ragged_buffer::BinOpSub>(&rhs.0)
+    }
 }
 
 #[pyproto]
 impl<'p> PyMappingProtocol for RaggedBufferI64 {
-    fn __getitem__(&self, index: IndicesOrInt<'p>) -> PyResult<RaggedBufferI64> {
+    fn __getitem__(&self, index: MultiIndex<'p>) -> PyResult<RaggedBufferI64> {
         match index {
-            IndicesOrInt::Indices(indices) => Ok(RaggedBufferI64(self.0.swizzle(indices)?)),
-            IndicesOrInt::Int(i) => Ok(RaggedBufferI64(self.0.get(i))),
+            MultiIndex::Index1(index) => match index {
+                Index::PermutationNP(indices) => Ok(RaggedBufferI64(self.0.swizzle(indices)?)),
+                Index::Permutation(_indices) => panic!("oh no"), //Ok(RaggedBufferI64(self.0.swizzle(indices)?)),
+                Index::Int(i) => Ok(RaggedBufferI64(self.0.get_sequence(i)?)),
+                Index::Slice(slice) => panic!("{:?}", slice),
+            },
+            MultiIndex::Index3((i0, i1, i2)) => Ok(RaggedBufferI64(Python::with_gil(|py| {
+                self.0.get_slice(py, i0, i1, i2)
+            })?)),
+            x => panic!("{:?}", x),
         }
     }
 }
