@@ -2,16 +2,16 @@ use numpy::{PyReadonlyArray1, PyReadonlyArray2, PyReadonlyArray3, ToPyArray};
 use std::cmp::Ordering;
 use std::collections::{binary_heap, BinaryHeap};
 use std::fmt::{Display, Write};
-use std::ops::{Add, Mul, Range};
+use std::ops::{Add, Mul, Range, Sub};
 
 use pyo3::prelude::*;
 
 #[derive(Clone, PartialEq, Eq, Hash, Debug)]
 pub struct RaggedBuffer<T> {
-    data: Vec<T>,
-    subarrays: Vec<Range<usize>>,
-    features: usize,
-    items: usize,
+    pub(crate) data: Vec<T>,
+    pub(crate) subarrays: Vec<Range<usize>>,
+    pub(crate) features: usize,
+    pub(crate) items: usize,
 }
 
 pub trait BinOp<T> {
@@ -27,6 +27,15 @@ impl<T: Add<T, Output = T>> BinOp<T> for BinOpAdd {
     }
 }
 
+pub struct BinOpSub;
+
+impl<T: Sub<T, Output = T>> BinOp<T> for BinOpSub {
+    #[inline]
+    fn op(lhs: T, rhs: T) -> T {
+        lhs - rhs
+    }
+}
+
 pub struct BinOpMul;
 
 impl<T: Mul<T, Output = T>> BinOp<T> for BinOpMul {
@@ -519,12 +528,16 @@ impl<T: numpy::Element + Copy + Display + std::fmt::Debug> RaggedBuffer<T> {
         ))
     }
 
-    pub fn len(&self) -> usize {
+    pub fn items(&self) -> usize {
         self.items
     }
 
+    pub fn len(&self) -> usize {
+        self.data.len()
+    }
+
     pub fn is_empty(&self) -> bool {
-        self.items == 0
+        self.data.is_empty()
     }
 }
 
